
#ifndef _DPMI_H_
  #define _DPMI_H_

  #ifdef __cplusplus
  extern "C" {
  #endif

  struct rmilong {
      long edi;
      long esi;
      long ebp;
      long reserved_by_system;
      long ebx;
      long edx;
      long ecx;
      long eax;
      short flags;
      short es,ds,fs,gs,ip,cs,sp,ss;
  } ;

  struct rmiword {
      short di, edi, si, esi;
      short bp, ebp;
      long reserved_by_system;
      short bx, ebx;
      short dx, edx;
      short cx, ecx;
      short ax, eax;
      short flags;
      short es,ds,fs,gs,ip,cs,sp,ss;
  } ;

  struct rmibyte {
      long edi, esi;
      long ebp;
      long reserved_by_system;
      char bl, bh;
      short ebx;
      char dl, dh;
      short edx;
      char cl, ch;
      short ecx;
      char al,ah;
      short eax;
      short flags;
      short es,ds,fs,gs,ip,cs,sp,ss;
  } ;

  extern int DPMI_allocCount;
  extern int DPMI_freeCount;
  extern long DPMI_allocated;

  extern unsigned long DPMI_maxAvail;
  extern void DPMI_init (void);

  extern void DPMI_debugInt (void);
  extern void DPMI_reboot (int warm);
  extern int DPMI_getES (void);
  extern int DPMI_getDS (void);
  extern void DPMI_setDS (int seg);
  extern void DPMI_setES (int seg);

  #define REBOOT_WARM 0x1234
  #define REBOOT_COLD 0

  #define PIC 0x20
  #define END_OF_PIC 0x20
  #define PIC_IRQ 0x21

  extern short DPMI_newSelector (long address, long size);


  #ifndef __TURBOC__

    #define INTERRUPT_ARG void

    #define DPMI_enableInt _enable
    #define DPMI_disableInt _disable

    #include <i86.h>

    union RMINFO {
        struct rmilong x;
        struct rmiword w;
        struct rmibyte h;
    };
      extern void *DPMI_alloc (unsigned short size);

//      extern void *MK_DP (unsigned short seg, unsigned short off);
//      extern unsigned short DP_SEG (void *ptr);
//      extern unsigned short DP_OFF (void *ptr);

      #define MK_DP(seg,off) (void *)((((unsigned short)seg)*16)+(unsigned short)off)
      #define DP_OFF(ptr) ((unsigned short)(FP_OFF(ptr)%16))
      #define DP_SEG(ptr) ((unsigned short)(FP_OFF(ptr)/16))

      extern void DPMI_free (void *dosPtr);

      extern long DPMI_getRealModeVector (int interruptNumber);
      extern void DPMI_setRealModeVector (int interruptNumber, long handler);

  #else

    #include "cgdk.h"

    extern void DPMI_disableInt (void);
    extern void DPMI_enableInt (void);

    #define INTERRUPT_ARG ...

    union RMINFO {
        struct rmiword x;
        struct rmibyte h;
    };


    #ifndef __LARGE__
      #error CGDK for Turbo C++ supports only LARGE memory model
    #endif

    #include <alloc.h>
    #include <dos.h>
    #include <process.h>

    #define MK_DP MK_FP
    #define DP_SEG FP_SEG
    #define DP_OFF FP_OFF
    #define int386x int86x
    #define int386 int86
    #define w x
    #define eax ax
    #define ebx bx
    #define ebp bp
    #define ecx cx
    #define edx dx
    #define esi si
    #define edi di
//    #define malloc farmalloc
//    #define calloc farcalloc
//    #define realloc farrealloc
//    #define free farfree
    #define DPMI_alloc _alloc
    #define DPMI_free _free
  #endif

  #ifdef __cplusplus
  }
  #endif

  extern void DPMI_int (char intNum, union RMINFO *rmi);

#endif
