
/* NET.H   IPX network library to create and control network games */
/* Copyright (C) 1996 Cerberus Electronics                         */
/* Version 0.1 beta                                                */
/* For Borland C++ 3.1/Large and Watcom C++ / DOS4GW               */
/* See an example application NETTEST.CPP                          */

#ifndef _NET_H
  #define _NET_H

  #include "ipxnode.h"

  #ifdef __cplusplus
  extern "C" {
  #endif

  #define NET_MAX_USERS 10       // the maximum number of logged users for one workgroup
  #define NET_MAX_WORKGROUPS 20  // maximum number of active workgroups

  #define NET_USER_STATUS_FREE 0
  #define NET_USER_STATUS_LOGGED 1
//  #define NET_STATUS_USER_CLEARED 2

  #define NET_STATUS_NOTHING 0
  #define NET_STATUS_LOGIN 1     // login mode - user can create, join or logout
  #define NET_STATUS_STARTUP 2    // starting up mode (no more login)
  #define NET_STATUS_SESSION 3        // running mode (no more login)
//  #define NET_STATUS_CLEARED 3     // a user connection has been cleared by the master

  #define NET_UPDATE_CLEARED_U 128
  #define NET_UPDATE_CLEARED 128
  #define NET_UPDATE_LOGOUT_MASTER 256
  #define NET_UPDATE_LOGOUT 256
  #define NET_UPDATE_LOGIN_MASTER 512
  #define NET_UPDATE_LOGIN 512
  #define NET_UPDATE_MISCINFO 1
  #define NET_UPDATE_NOTHING 0
  #define NET_UPDATE_NEWWORKGROUP 110
  #define NET_UPDATE_UPDATEWORKGROUP 111
  #define NET_UPDATE_DELETEWORKGROUP 112
  #define NET_UPDATE_LOGINACCESSDENIED 150
  #define NET_UPDATE_NONETWORK 160

  #define NET_NODE_MASTER 1      // the node to communicate with the master
  #define NET_USER_MASTER 0

  #define NET_LOGIN_ENABLE 0
  #define NET_LOGIN_DISABLE 1

  #define NET_LOGINRESULT_SUCCESS 0
  #define NET_LOGINRESULT_ALREADYLOGGED 1
  #define NET_LOGINRESULT_INVALIDGROUP 2
  #define NET_LOGINRESULT_ACCESSDENIED 3
  #define NET_LOGINRESULT_LOGINDISABLED 4
  #define NET_LOGINRESULT_TOOMANYWORKGROUPS 5
  #define NET_LOGINRESULT_TOOMANYUSERS 6
  #define NET_LOGINRESULT_TIMEOUT 7
  #define NET_LOGINRESULT_NONETWORK 8
  #define NET_LOGINRESULT_NOTLOGINMODE 9
  #define NET_LOGINRESULT_GROUPALREADYEXISTS 10
  #define NET_LOGINRESULT_BADNAME 11

  #define NET_PASSWORD_NONE 0
  #define NET_PASSWORD_REQUIRED 1


  #define NET_MAXLEN_NAME 20
  #define NET_MAXLEN_DESC 50
  #define NET_MAXLEN_PASSWORD 20

//  #define NET_WORKGROUP_EXTERNAL 0
//  #define NET_WORKGROUP_LOCAL 1

  typedef struct
  {
    char Name [NET_MAXLEN_NAME+1];          // here is the member name
    char Status;             // NET_USER_STATUS
    TIPXAddress Addr;         // here is the address of the user
  } TNetUser;

  // this structure is always broadcasted only before startup
  typedef struct
  {
    char       Desc [NET_MAXLEN_DESC+1];       // description of current net game (Anonymous deathmatch)

    char       UsersCountMax;   // the maximum count of logged users
    char       Status;          // should be running or login
    long       TimeStamp;       // time stamp
    char       Login;           // login enabled/disabled
    char       Password;        // password needed for login
    char       User;            // 0 = master 1 .. n = number of slave
    char       UsersCount;      // current number of members in node
    TNetUser   Users [NET_MAX_USERS]; // all users. 0 is master

  } TNetWorkGroup;                 // only main computer is sending this data


  extern TNetWorkGroup NetLocalWorkGroup;
  extern TNetWorkGroup NetWorkGroup [NET_MAX_WORKGROUPS]; // all work groups data; 0-th is local work group

  // initializises NET library
  int NET_Init (void);

  // deinitializes NET library
  void NET_Done (void);

  int NET_Send2User (char *user, void *data, int dataSize);

  // send data to specifed node (after login)
  int NET_Send (int NodeNumber, void *Data, int DataSize);

  // receive data from specified node (after login)
  int NET_Receive (int NodeNumber, void *Data, int DataSize);

  // send data to specifed node (after login)
  int NET_SendEveryone (void *Data, int DataSize);

  // receive data from specified node (after login)
  int NET_ReceiveEveryone (void *Data, int DataSize);

  // scans for all work groups over the net and fills WorkGroup [] data
//  int NET_ScanWorkGroups (void);

  // returns the pointer to specified net workgroup info
  TNetWorkGroup *NET_GetWorkGroupInfo (int Number);

  // updates the net. receives new commands. call at least every second !
  int NET_Update (void);

  // this function is available only for the master computer
  int NET_Startup (void);

  // call this function after status is set to NET_STARTUP when you are ready
  int NET_Synchronize (void);

  // leave the network game
  int NET_Logout (void);

  // enable login
  int NET_EnableLogin (void);

  // kill the slave
  int NET_ClearSlave (int User);

  // disable login
  int NET_DisableLogin (void);

  // create a new workgroup
  int NET_Create (const char *GameDesc, const char *PlayerName, const char *Password);

  // join a workgroup
  int NET_Join (const char *GameDesc,  const char *PlayerName, const char *Password);

  int NET_JoinAddress (TIPXAddress *address, const char *PlayerName, const char *Password);

  // a master can set the maximum number of logged users
  // use best before first NET_Update ()
  int NET_SetMaxUsers (int MaxUsers);

  int NET_GetWorkGroupsCount (void);

  char *NET_WhatsMyName (void);

  int NET_findWorkgroup (char *name);

  #ifdef __cplusplus
  }
  #endif

#endif
